#ifndef __TabStackP_h__
#define __TabStackP_h__

#if defined(VMS) || defined(__VMS)
#include <X11/apienvset.h>
#endif

#include <Xm/XmP.h>
#include <Xm/ManagerP.h>
#include <Xm/BulletinBP.h>
#include <Xi/XiP.h>
#include <Xi/TabBox.h>
#include <Xi/TabList.h>
#include <Xi/TabStack.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _XiTabStackClassPart {
    String    drag_translations;
    XtPointer extension;
} XiTabStackClassPart;

typedef struct _XiTabStackClassRec {
    CoreClassPart            core_class;
    CompositeClassPart	     composite_class;
    ConstraintClassPart      constraint_class;
    XmManagerClassPart       manager_class;
    XmBulletinBoardClassPart bulletin_board_class;
    XiTabStackClassPart      tab_stack_class;
} XiTabStackClassRec;

typedef struct _XiTabStackPart {
    /* Resources for the Tab Stack */

    XtCallbackList       tab_select_callback;

    /* Resource to pass down to Tab Box */
    XmFontList		 font_list;
    XiTabStyle		 tab_style;
    XiTabMode		 tab_mode;
    XiTabSide            tab_side;
    XiTabOrientation	 tab_orientation;

    XmString		 tear_off_label;

    Boolean		 allow_tear_offs;
    Boolean		 uniform_tab_size;
    Boolean		 use_image_cache;
    Boolean		 stacked_effect;
    Boolean		 tab_auto_select;
    
    Dimension            tab_margin_width;
    Dimension            tab_margin_height;
    Dimension		 tab_label_spacing;
    Dimension            tab_offset;
    Dimension            highlight_thickness;

    Pixel		 select_color;
    Pixel		 select_pixmap;

    int			 tab_corner_percent;

    Widget               tab_box;

    /* Private Values */

    XiTabList		 _tab_list;

    XRectangle           _size;

    Widget               _active_child;
    GC			 _gc;

    Boolean		 _inited;
    Boolean		 _set_tab_list;

    Widget               _menu;
    Widget		 _tear_off_button;

    /* Drag And Drop Stuff */
    Pixmap               _source_pixmap;
    Pixmap               _source_mask;
    Pixmap               _invalid_pixmap;
    Pixmap               _invalid_mask;
    Widget               _source_icon;
    Widget               _invalid_icon;

    Widget		_selected_tab;	    /* used within realize method */
    Boolean		_selected_notify;   /* used within realize method */

    Boolean		do_notify;	/* for notify XmNtabSelectedCallback */

} XiTabStackPart;

typedef struct _XiTabStackRec {
    CorePart             core;
    CompositePart        composite;
    ConstraintPart       constraint;
    XmManagerPart        manager;
    XmBulletinBoardPart  bulletin_board;
    XiTabStackPart       tab_stack;
} XiTabStackRec;

typedef struct _XiTabStackConstraintPart {
    XmString    	 tab_label_string;
    XmStringDirection    tab_string_direction;
    unsigned char        tab_alignment;
    Pixmap		 tab_label_pixmap;
    XiPixmapPlacement    tab_pixmap_placement;
    Pixel		 tab_foreground;
    Pixel		 tab_background;
    Pixmap		 tab_background_pixmap;
    Boolean		 free_tab_pixmap;

    Boolean		 tear_off_enabled;

    /* Private Memebers */
    int			index;
    Dimension		width;
    Dimension		height;
} XiTabStackConstraintPart;

typedef struct _XiTabStackConstraintRec {
    XmManagerConstraintPart  manager;
    XiTabStackConstraintPart tab_stack;
} XiTabStackConstraintRec, * XiTabStackConstraintPtr;

#define _XiC(w) ((XiTabStackConstraintPtr)((w)->core.constraints))->tab_stack
#define _XiCtabLabelString(w) _XiC(w).tab_label_string
#define _XiCtabAlignment(w) _XiC(w).tab_alignment
#define _XiCtabStringDirection(w) _XiC(w).tab_string_direction
#define _XiCtabLabelPixmap(w) _XiC(w).tab_label_pixmap
#define _XiCtabPixmapPlacement(w) _XiC(w).tab_pixmap_placement
#define _XiCtabForeground(w) _XiC(w).tab_foreground
#define _XiCtabBackground(w) _XiC(w).tab_background
#define _XiCtabBackgroundPixmap(w) _XiC(w).tab_background_pixmap
#define _XiCwidth(w) _XiC(w).width
#define _XiCheight(w) _XiC(w).height
#define _XiCtearOffEnabled(w) _XiC(w).tear_off_enabled
#define _XiCfreePixmap(w) _XiC(w).free_tab_pixmap
#define _XiCindex(w) _XiC(w).index

#define XiNillegalUniformTabSizeMsg \
"XmNuniformTabSize must be true if XmNtabMode is XiTABS_STACKED or\n\
XiTABS_STACKED_STATIC."

#ifdef __cplusplus
} /* Close scope of 'extern "C"' declaration */
#endif

#if defined(VMS) || defined(__VMS)
#include <X11/apienvrst.h>
#endif

#endif /* __TabStackP_h__ */
