#ifndef _IGBM_
#define _IGBM_

/* NOSHIP */
#ifndef _IGRAPHPL_H
#include <igraphpl.hpp>
#endif

#include <ibase.hpp>
#ifdef IC_WIN
#include <windows.h>
#endif // IC_WIN
#ifdef IC_PM
#define INCL_DOS
#define INCL_WIN
#define INCL_DOSPROCESS

#define INCL_GPI
#define INCL_DEV
#define INCL_BITMAPFILEFORMAT
#include <os2.h>
#endif // IC_PM
#ifdef IC_MOTIF
#define BYTE unsigned char
#define CHAR char
#endif // IC_MOTIF

/* NOSHIP */
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 57 1.5.1.4 source/ographic/igbm.hpp, graphics, ioc.v400, 000920 
/*******************************************************************************
* FILE NAME: igbmstnd.h                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class(es):                                              *
*     IGBMSTND.H  Custom header file that provides commonly used equates       *
*                                                                              *
*                                                                              *
*******************************************************************************/

#ifndef STANDARD_H
#define STANDARD_H

#if 0
//typedef int BOOLEAN;
#define TRUE            1
#define FALSE           0
#endif // 0

#define byte            unsigned char
#define word            unsigned short
#define dword           unsigned long

#endif // STANDARD_H
/* NOSHIP */
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
//
//
/*******************************************************************************
* FILE NAME: igbm.h                                                            *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class(es):                                              *
*   GBM.H  General Bitmap Code                                                 *
*                                                                              *
*   Data is stored as an array of lines.                                       *
*   Lines are stored with bottom line first, moving upwards.                   *
*   Each line is an array of pixels, leftmost first.                           *
*   Lines are padded to be a multiple of a dword long.                         *
*   Palettised pixels are either a 1 bit, 4 bit, or 8 bit indexes.             *
*   Alternately a B, G, R triple in that order is stored.                      *
*   This format exactly matches the format used by OS/2 and Windows bitmaps.   *
*                                                                              *
*   One notable point: a 1 in a 1bpp image denotes colour 1, as found by       *
*   looking at palette entry 1. Data is not inverse when passed to and from    *
*   GBM.                                                                       *
*                                                                              *
*******************************************************************************/

typedef int GBM_ERR;
#define GBM_ERR_OK              ((GBM_ERR) 0)
#define GBM_ERR_MEM             ((GBM_ERR) 1)
#define GBM_ERR_NOT_SUPP        ((GBM_ERR) 2)
#define GBM_ERR_BAD_OPTION      ((GBM_ERR) 3)
#define GBM_ERR_NOT_FOUND       ((GBM_ERR) 4)
#define GBM_ERR_BAD_MAGIC       ((GBM_ERR) 5)
#define GBM_ERR_BAD_SIZE        ((GBM_ERR) 6)
#define GBM_ERR_READ            ((GBM_ERR) 7)
#define GBM_ERR_WRITE           ((GBM_ERR) 8)

#define GBM_FT_R1               0x0001
#define GBM_FT_R4               0x0002
#define GBM_FT_R8               0x0004
#define GBM_FT_R24              0x0008
#define GBM_FT_W1               0x0010
#define GBM_FT_W4               0x0020
#define GBM_FT_W8               0x0040
#define GBM_FT_W24              0x0080

typedef struct
        {
        char *short_name;               /* Eg: "Targa"                       */
        char *long_name;                /* Eg: "Truevision Targa / Vista"    */
        char *extensions;               /* Eg: "TGA VST"                     */
        int flags;                      /* What functionality exists         */
        } GBMFT;

typedef struct { byte r, g, b; } GBMRGB;

#define PRIV_SIZE 2000

typedef struct
        {
        int w, h, bpp;                  /* Bitmap dimensions                 */
        byte priv [PRIV_SIZE];          /* Private internal buffer           */
        } GBM;

#if 0
#ifndef IC_MOTIF
#ifndef _GBM_

GBM_ERR gbm_init(void);
GBM_ERR gbm_deinit(void);

GBM_ERR gbm_query_n_filetypes(int *n_ft);
GBM_ERR gbm_query_filetype(int ft, GBMFT *gbmft);
GBM_ERR gbm_guess_filetype(char *fn, int *ft);

GBM_ERR gbm_read_header(char *fn, int fd, int ft, GBM *gbm, char *opt);
GBM_ERR gbm_read_palette(int fd, int ft, GBM *gbm, GBMRGB *gbmrgb);
GBM_ERR gbm_read_data(int fd, int ft, GBM *gbm, byte *data);
GBM_ERR gbm_write(char *fn, int fd, int ft, GBM *gbm, GBMRGB *gbmrgb, byte *data, char *opt);
#endif
unsigned long gbm_err(GBM_ERR rc);



#endif //IC_MOTIF
#endif // 0
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 57 1.5.1.4 source/ographic/igbm.hpp, graphics, ioc.v400, 000920 
/*

MODEL.C  Work on the model used in GbmV2

*/

#ifndef MODEL_H
#define MODEL_H

#if 0
#include "igbmstnd.hpp"
#include "igbm.hpp"
#endif // 0

#define UNKNOWN_IMAGETYPE -1

/* These needs to encompass all the GBM errors, plus some extras */
typedef int MOD_ERR;
#define MOD_ERR_OK      ((MOD_ERR) 0)
#define MOD_ERR_MEM     ((MOD_ERR) 1)
#define MOD_ERR_OPEN    ((MOD_ERR) 2)
#define MOD_ERR_CREATE  ((MOD_ERR) 3)
#define MOD_ERR_SUPPORT ((MOD_ERR) 4)
#define MOD_ERR_HDC     ((MOD_ERR) 5)
#define MOD_ERR_HPS     ((MOD_ERR) 6)
#define MOD_ERR_HBITMAP ((MOD_ERR) 7)
#define MOD_ERR_HMF     ((MOD_ERR) 8)
#define MOD_ERR_CLIP    ((MOD_ERR) 9)
#define MOD_ERR_GBM(rc) ((MOD_ERR) 0x8000+(rc))

typedef struct _MOD
        {
        GBM    gbm;
        GBMRGB gbmrgb[0x100];
        BYTE*  pbData;
        long   bufSize;
        } MOD;

#if 0
extern "C" MOD_ERR ModCreate(
        int w, int h, int bpp, GBMRGB gbmrgb[],
        MOD *modNew
        );
#endif // 0

extern "C" MOD_ERR ModDelete(MOD *mod);

extern "C" MOD_ERR ModCopy(MOD *mod, MOD *modNew);

extern "C" MOD_ERR ModMove(MOD *mod, MOD *modNew);

extern "C" MOD_ERR ModCreateFromFile( CHAR *szFn,
                                      int *ft,
                                      CHAR *szOpt,
                                      MOD *modNew );

#ifndef IC_MOTIF
extern "C" MOD_ERR ModCreateFromHBitmap( MOD     *mod,
#ifdef IC_WIN
                                         HBITMAP  hbm,
                                         HPALETTE hpal );
#endif
#ifdef IC_PM
                                         HAB     hab,
                                         HBITMAP hbm );
#endif
#endif // IC_MOTIF

extern "C" MOD_ERR ModWriteToFile( MOD *mod,
                                   int ft,
                                   CHAR *szFn,
                                   CHAR *szOpt );

#if 0
extern "C" MOD_ERR ModExpandTo24Bpp(MOD *mod, MOD *mod24);
#endif // 0

extern "C" MOD_ERR ModReflectHorz(MOD *mod, MOD *modNew);
extern "C" MOD_ERR ModReflectVert(MOD *mod, MOD *modNew);
extern "C" MOD_ERR ModTranspose  (MOD *mod, MOD *modNew);
extern "C" MOD_ERR ModRotate90   (MOD *mod, MOD *modNew);
extern "C" MOD_ERR ModRotate180  (MOD *mod, MOD *modNew);
extern "C" MOD_ERR ModRotate270  (MOD *mod, MOD *modNew);

#if 0
extern "C" MOD_ERR ModExtractSubrectangle(
        MOD *mod,
        int x, int y, int w, int h,
        MOD *modNew
        );

extern MOD_ERR ModBlit(
        MOD *modDst, int dx, int dy,
        MOD *modSrc, int sx, int sy,
        int w, int h
        );

/*...scolour adjustment mappings:0:*/
#define CVT_I_TO_L      0
#define CVT_I_TO_P      1
#define CVT_L_TO_I      2
#define CVT_L_TO_P      3
#define CVT_P_TO_I      4
#define CVT_P_TO_L      5
/*...e*/

extern "C" MOD_ERR ModColourAdjust(
        MOD *mod,
        int map, double gama, double shelf,
        MOD *modNew
        );
#endif // 0

/*...sbpp palette and algorithm mappings:0:*/
#define CVT_BW          0
#define CVT_VGA         1
#define CVT_8           2
#define CVT_4G          3
#define CVT_784         4
#define CVT_666         5
#define CVT_8G          6
#define CVT_TRIPEL      7
#define CVT_RGB         8
#define CVT_FREQ        9
#define CVT_MCUT        10

#define CVT_NEAREST     0
#define CVT_ERRDIFF     1
#define CVT_HALFTONE    2
/*...e*/

#if 0
extern "C" MOD_ERR ModBppMap(
        MOD *mod,
        int iPal, int iAlg,
        int iKeepRed, int iKeepGreen, int iKeepBlue, int nCols,
        MOD *modNew
        );
#endif // 0

extern "C" MOD_ERR ModResize(
        MOD *mod,
        int nw, int nh,
        MOD *modNew
        );

#if 0
extern "C" MOD_ERR ModMakeHBITMAP(
        MOD *mod,
#ifdef IC_WIN
        HBITMAP *phbm,
        HPALETTE *phpal
#endif
#ifdef IC_PM
        HAB hab,
        HBITMAP *phbm
#endif
        );
#endif // 0

extern "C" bool ModMapVisual(
#ifdef IC_WIN
        MOD *mod, int viewOption, HBITMAP *phbm,
        LONG *plColorBg, LONG *plColorFg, HPALETTE *phpal
#endif
#ifdef IC_PM
        HAB hab, MOD *mod, int viewOption, HBITMAP* phbm,
        LONG *plColorBg, LONG *plColorFg
#endif
        );

extern "C" unsigned long ModError(MOD_ERR rc);

#ifdef IC_WIN
  extern "C" HPALETTE CreateDIBPal(LPBITMAPINFO pbmi, bool bCoreHdr, int *iNumClr);

  extern "C" HBITMAP LoadBitmapResource( HINSTANCE hInstance, LPSTR lpString, HPALETTE *lphPalette);
#endif

#endif // MODEL_H
#ifdef IC_MOTIF
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 57 1.5.1.4 source/ographic/igbm.hpp, graphics, ioc.v400, 000920 
/*

IGBMMIR.H  Interface to Mirror Image of General Bitmap

*/

extern bool gbm_ref_vert(GBM *gbm, byte *data);
extern bool gbm_ref_horz(GBM *gbm, byte *data);
extern void gbm_transpose(GBM *gbm, byte *data, byte *data_t);
/* NOSHIP */
/*
*****************************************************************************************
*                                                                                       *
* COPYRIGHT:                                                                            *
*   IBM Open Class Library                                                              *
*   (C) Copyright International Business Machines Corporation,  1997                    *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.                  *
*   US Government Users Restricted Rights - Use, duplication, or disclosure             *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                              *
*                                                                                       *
*****************************************************************************************
*/
// Revision: 57 1.5.1.4 source/ographic/igbm.hpp, graphics, ioc.v400, 000920 
#ifndef _IGBSCAL_
#define _IGBSCAL_
/*******************************************************************************
* FILE NAME: igbmscal.h                                                        *
*                                                                              *
* DESCRIPTION:                                                                 *
*   Declaration of the class(es):                                              *
*     IGBMSCAL.H  Interface to scaling code                                    *
*                                                                              *
*                                                                              *
*******************************************************************************/

extern GBM_ERR gbm_simple_scale(
        byte *s, int sw, int sh,
        byte *d, int dw, int dh,
        int bpp
        );

#endif
#endif // IC_MOTIF
#endif // IGBM header guard
