/*******************************************************************************
* FILE NAME: igraphic.cpp                                                      *
*                                                                              *
* DESCRIPTION:                                                                 *
*   This file contains the implementation of classes/functions declared        *
*   in igraphic.cpp.                                                           *
*                                                                              *
* COPYRIGHT:                                                                   *
*   IBM Open Class Library                                                     *
*   (C) Copyright International Business Machines Corporation 1992, 1997       *
*   Licensed Material - Program-Property of IBM - All Rights Reserved.         *
*   US Government Users Restricted Rights - Use, duplication, or disclosure    *
*   restricted by GSA ADP Schedule Contract with IBM Corp.                     *
*                                                                              *
*******************************************************************************/
// Revision: 64 1.3.1.3 source/ographic/igraphic.cpp, graphics, ioc.v400, 000920 

#include <ibase.hpp>

#if ( IC_OBSOLETE <= IC_OBSOLETE_3 )
#ifdef IC_PMWIN

#include <igraphic.hpp>
#include <igrafctx.hpp>
#include <ipoint.hpp>

// Segment definitions
#ifdef IC_PAGETUNE
  #define _IGRAPHIC_CPP_
  #include <ipagetun.h>
#endif

/*------------------------------------------------------------------------------
| IGraphic::IGraphic                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGraphic::IGraphic( )
  : fId(0), fTm(IGraphic::rightMultiply), fBundle(0), fTransMat(0),
    fSelected(false), fSelectable(true), fIsList(false)
{
}

/*------------------------------------------------------------------------------
| IGraphic::IGraphic                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGraphic::IGraphic( const IGraphicBundle& graphicBundle )
  : fId(0), fTm(IGraphic::rightMultiply), fBundle(0), fTransMat(0),
    fSelected(false), fSelectable(true), fIsList(false)
{
  fBundle = new IGraphicBundle(graphicBundle);
}

/*------------------------------------------------------------------------------
| IGraphic::IGraphic                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGraphic::IGraphic( const IGraphic& graphic )
  : fBundle(0), fTransMat(0)
{
  if (graphic.hasGraphicBundle())
    this->setGraphicBundle(graphic.graphicBundle());
  if (graphic.hasTransformMatrix())
    this->setTransformMatrix(graphic.transformMatrix());

  this->fId         = graphic.fId;
  this->fTm         = graphic.fTm;
  this->fSelected   = graphic.fSelected;
  this->fSelectable = graphic.fSelectable;
  this->fIsList     = graphic.fIsList;
}

/*------------------------------------------------------------------------------
| IGraphic::~IGraphic                                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGraphic::~IGraphic( )
{
  if ( fBundle )
  {
    delete fBundle;
  }
  if ( fTransMat )
  {
    delete fTransMat;
  }
}

/*------------------------------------------------------------------------------
| IGraphic::boundingRect                                                       |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IRectangle IGraphic::boundingRect( IGraphicContext& graphicContext )
{
  graphicContext.startBoundaryAccumulation();
  drawOn( graphicContext );
  return graphicContext.stopBoundaryAccumulation();
}

/*------------------------------------------------------------------------------
| IGraphic::setGraphicBundle                                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGraphic& IGraphic::setGraphicBundle( const IGraphicBundle& graphicBundle )
{
  if ( fBundle )
    *fBundle = graphicBundle;
  else
    fBundle = new IGraphicBundle(graphicBundle);
  return *this;
}

/*------------------------------------------------------------------------------
| IGraphic::graphicBundle                                                      |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGraphicBundle IGraphic::graphicBundle ( ) const
{
  if (fBundle)
    return IGraphicBundle(*fBundle);
  else
    return IGraphicBundle();
}

/*------------------------------------------------------------------------------
| IGraphic::removeGraphicBundle                                                |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGraphic& IGraphic::removeGraphicBundle( )
{
  if ( fBundle )
  {
    delete fBundle;
    fBundle = 0;
  }
  return *this;
}

/*------------------------------------------------------------------------------
| IGraphic::rotateBy                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGraphic& IGraphic::rotateBy( double angle, const IPoint &pt )
{
  if (!fTransMat)
    fTransMat = new ITransformMatrix;

  fTransMat->rotateBy( angle, pt );

  return *this;
}

/*------------------------------------------------------------------------------
| IGraphic::scaleBy                                                            |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGraphic& IGraphic::scaleBy( double xScale, double yScale, const IPoint& pt )
{
  if (!fTransMat)
    fTransMat = new ITransformMatrix;

  fTransMat->scaleBy( xScale, yScale, pt );

  return *this;
}

/*------------------------------------------------------------------------------
| IGraphic::translateBy                                                        |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGraphic& IGraphic::translateBy( const IPoint& pt )
{
  if (!fTransMat)
    fTransMat = new ITransformMatrix;

  fTransMat->translateBy( pt );

  return *this;
}

/*------------------------------------------------------------------------------
| IGraphic::setTransformMatrix                                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGraphic& IGraphic::setTransformMatrix( const ITransformMatrix& transformMatrix)
{
  if (fTransMat)
    *fTransMat = transformMatrix;
  else
    fTransMat = new ITransformMatrix( transformMatrix );

  return *this;
}

/*------------------------------------------------------------------------------
| IGraphic::resetTransformMatrix                                               |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGraphic& IGraphic::resetTransformMatrix()
{
  if ( fTransMat )
  {
    delete fTransMat;
    fTransMat = 0;
  }
  return *this;
}


/*------------------------------------------------------------------------------
| IGraphic::transformMatrix                                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
const ITransformMatrix IGraphic::transformMatrix() const
{
  if ( fTransMat )
    return ITransformMatrix( *fTransMat );
  else
    return ITransformMatrix();
}

/*------------------------------------------------------------------------------
| IGraphic::contains                                                           |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
bool IGraphic::contains( IGraphicContext& graphicContext, const IPoint& pt )
{
  setHitSelected( false );

  graphicContext.startHitTesting()
                .setHitPoint(pt);
  drawOn( graphicContext );
  graphicContext.stopHitTesting();

  return isHitSelected();
}

/*------------------------------------------------------------------------------
| IGraphic::setTransformMethod                                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGraphic& IGraphic::setTransformMethod( TransformMethod transformMethod )
{
  fTm = transformMethod;
  return *this;
}

/*------------------------------------------------------------------------------
| IGraphic::setTransformMethod                                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGraphic::TransformMethod IGraphic::transformMethod( ) const
{
  return fTm;
}

/*------------------------------------------------------------------------------
| IGraphic::hasGraphicBundle                                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
bool IGraphic::hasGraphicBundle() const
{
  return ((fBundle == 0) ? false : true);
}

/*------------------------------------------------------------------------------
| IGraphic::hasTransformMatrix                                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
bool IGraphic::hasTransformMatrix() const
{
  return ((fTransMat == 0) ? false : true);
}

/*------------------------------------------------------------------------------
| IGraphic::setId                                                              |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGraphic& IGraphic::setId( unsigned long identifier )
{
  fId = identifier;
  return *this;
}

/*------------------------------------------------------------------------------
| IGraphic::id                                                                 |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
unsigned long IGraphic::id( ) const
{
  return fId;
}

/*------------------------------------------------------------------------------
| IGraphic::setHitSelected                                                     |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGraphic& IGraphic::setHitSelected( bool hitSelected )
{
  this->fSelected = hitSelected;
  return *this;
}

/*------------------------------------------------------------------------------
| IGraphic::isHitSelected                                                      |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
bool IGraphic::isHitSelected( ) const
{
  return (fSelectable ? fSelected : false);
}

/*------------------------------------------------------------------------------
| IGraphic::setHitSelectable                                                   |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGraphic& IGraphic::setHitSelectable( bool hitSelectable )
{
  this->fSelectable = hitSelectable;
  return *this;
}

/*------------------------------------------------------------------------------
| IGraphic::isHitSelectable                                                    |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
bool IGraphic::isHitSelectable( ) const
{
  return fSelectable;
}

/*------------------------------------------------------------------------------
| IGraphic::operator=                                                          |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
IGraphic& IGraphic::operator= ( const IGraphic& graphic )
{
  if ( this == &graphic )
    return *this;

  this->removeGraphicBundle();
  this->resetTransformMatrix();

  if (graphic.hasGraphicBundle())
    this->setGraphicBundle(graphic.graphicBundle());
  if (graphic.hasTransformMatrix())
    this->setTransformMatrix(graphic.transformMatrix());

  this->fId         = graphic.fId;
  this->fTm         = graphic.fTm;
  this->fSelected   = graphic.fSelected;
  this->fSelectable = graphic.fSelectable;
  this->fIsList     = graphic.fIsList;

  return *this;
}

/*------------------------------------------------------------------------------
| IGraphic::operator==                                                         |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
bool IGraphic::operator== ( const IGraphic& graphic ) const
{
  if (( (this->hasGraphicBundle() && graphic.hasGraphicBundle()) ||
      (!(this->hasGraphicBundle()) && !(graphic.hasGraphicBundle())) ) &&
      ( (this->hasTransformMatrix() && graphic.hasTransformMatrix()) ||
      (!(this->hasTransformMatrix()) && !(graphic.hasTransformMatrix())) ) &&
      (this->transformMatrix() == graphic.transformMatrix()) &&
      (this->graphicBundle() == graphic.graphicBundle()) &&
      (this->fId == graphic.fId) &&
      (this->fTm == graphic.fTm) &&
      (this->fSelected == graphic.fSelected) &&
      (this->fSelectable == graphic.fSelectable))
    return true;
  else
    return false;
}

/*------------------------------------------------------------------------------
| IGraphic::operator!=                                                         |
|                                                                              |
|                                                                              |
------------------------------------------------------------------------------*/
bool IGraphic::operator!= ( const IGraphic& graphic ) const
{
  return !(*this == graphic);
}

#endif // IC_PMWIN
#endif // IC_OBSOLETE
